package cn.estsh.i3plus.ext.mes.pcn.api.busi;

import cn.estsh.i3plus.ext.mes.pcn.pojo.context.MesCellEquipContext;
import cn.estsh.i3plus.ext.mes.pcn.pojo.context.MesProdShiftContext;
import cn.estsh.i3plus.pojo.mes.model.StationKvBean;
import cn.estsh.i3plus.pojo.mes.model.StationRequestBean;
import cn.estsh.i3plus.pojo.mes.util.MesExtEnumUtil;
import io.swagger.annotations.ApiOperation;

import java.util.List;

/**
 * 获取生产过程上下文对象接口【BUSI】
 */
public interface IMesProductionCustomContextStepService {

    @ApiOperation(value = "验证上下文工位加工锁是否存在")
    Boolean checkProductionLockIsExistContext(StationRequestBean reqBean);

    @ApiOperation(value = "保存上下文加工锁")
    Boolean saveProductionLockContext(StationRequestBean reqBean, String value);

    @ApiOperation(value = "删除上下文加工锁")
    void deleteProductionLockContext(StationRequestBean reqBean);

    @ApiOperation(value = "发送设备质量信息")
    void sendEquipQualityMessage(StationRequestBean reqBean, MesCellEquipContext cellEquipContext);

    @ApiOperation(value = "存储上个扫描/读取信息")
    Boolean saveLastScanContext(StationRequestBean reqBean, List<StationKvBean> dataList);

    @ApiOperation(value = "获取上个扫描/读取信息")
    List<StationKvBean> getLastScanContext(StationRequestBean reqBean);

    @ApiOperation(value = "发送工步内容")
    void doSendStepContextMessage(StationRequestBean reqBean);

    @ApiOperation(value = "发送工步内容")
    void doSendStepContextMessage(StationRequestBean reqBean, String scanInfo, MesExtEnumUtil.CELL_MESSAGE_SOURCE source);

    @ApiOperation(value = "发送工步内容")
    void sendStepContextMessage(StationRequestBean reqBean, List<StationKvBean> resultList);

    @ApiOperation(value = "获取手动选择的腔数及工单信息上下文")
    List<StationKvBean> getFunctionChooseCavityOrderContext(StationRequestBean reqBean);

    @ApiOperation(value = "保存选择/扫描/读取的腔数及工单信息上下文", notes = "[StationKvBean(key=cavity, name=腔数, value=3, seq=0), StationKvBean(key=cellMessageSource, name=工位信息来源, value=10, seq=1), StationKvBean(key=workOrderNo, name=加工单, value=工单1;工单2;工单3, seq=2)]")
    Boolean saveFunctionChooseCavityOrderContext(StationRequestBean reqBean, List<StationKvBean> resultList);

    @ApiOperation(value = "删除手动选择的腔数及工单信息上下文")
    void deleteFunctionChooseCavityOrderContext(StationRequestBean reqBean);

    @ApiOperation(value = "验证上下文班次班组信息是否存在")
    Boolean checkProdShiftDataIsExistContext(String orgainzeCode, String workCenterCode);

    @ApiOperation(value = "获取上下文班次班组信息")
    List<StationKvBean> getProdShiftDataContext(String orgainzeCode, String workCenterCode);

    @ApiOperation(value = "保存上下文班次班组信息", notes = "[JSON]List<StationKvBean>")
    Boolean saveProdShiftDataContext(String orgainzeCode, String workCenterCode, List<StationKvBean> prodShiftDatList);

    @ApiOperation(value = "删除上下文班次班组信息")
    void deleteProdShiftDataContext(String orgainzeCode, String workCenterCode);

    @ApiOperation(value = "获取上下文工位加工数统计信息")
    List<StationKvBean> getProductionStatisticsContext(StationRequestBean reqBean);

    @ApiOperation(value = "保存上下文工位加工数统计信息", notes = "[JSON]List<StationKvBean>")
    Boolean saveProductionStatisticsContext(StationRequestBean reqBean, List<StationKvBean> prodShiftDatList);

    @ApiOperation(value = "新增上下文工位加工数统计信息")
    Boolean addProductionStatisticsContext(StationRequestBean reqBean, int count);

    @ApiOperation(value = "删除上下文工位加工数统计信息")
    void deleteProductionStatisticsContext(StationRequestBean reqBean);

    @ApiOperation(value = "获取生产线的当前班组班次信息")
    MesProdShiftContext getMesProdShiftKvBean(String orgainzeCode, String workCenterCode);
}
