package cn.estsh.i3plus.ext.mes.pcn.apiservice.serviceimpl.step.context;

import cn.estsh.i3plus.ext.mes.pcn.api.busi.IMesProductionDispatchContextStepService;
import cn.estsh.i3plus.ext.mes.pcn.pojo.context.*;
import cn.estsh.i3plus.ext.mes.pcn.pojo.util.MesPcnExtConstWords;
import cn.estsh.i3plus.mes.pcn.serviceimpl.fsm.BaseStepService;
import cn.estsh.i3plus.mes.pcn.util.StationKvBeanUtil;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.mes.bean.MesPart;
import cn.estsh.i3plus.pojo.mes.bean.MesRawPartCharging;
import cn.estsh.i3plus.pojo.mes.model.StationKvBean;
import cn.estsh.i3plus.pojo.mes.model.StationRequestBean;
import cn.estsh.i3plus.pojo.mes.util.MesExtEnumUtil;
import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.TypeReference;
import com.google.common.base.Objects;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.StringJoiner;

/**
 * @Description : 获取生产过程上下文对象接口实现【BUSI】
 * @Author : wangjie
 **/
@Slf4j
@Service("mesProductionDispatchContextStepService")
public class MesProductionDispatchContextStepService extends BaseStepService implements IMesProductionDispatchContextStepService {

    private String getContextKey(StationRequestBean reqBean) { return getFsmBusikey(reqBean, new StringJoiner(MesPcnExtConstWords.COLON).add(MesPcnExtConstWords.PRODUCTION_PROCESS_CONTEXT).add(MesPcnExtConstWords.BUSI_DATA_CONTEXT).toString()); }

    private String getContextKey(String organizeCode, String busiCode) { return new StringJoiner(":").add(organizeCode).add(MesPcnExtConstWords.PRODUCTION_PROCESS_CONTEXT).add(busiCode).toString(); }

    //清除上下文中的所有业务数据
    @Override
    public void doFlushProductionDispatchContext(StationRequestBean reqBean) {
        deleteFsmBusiData(reqBean.getOrganizeCode(), getContextKey(reqBean));
    }

    //存储展示组件MODULE_CONTENT内容
    @Override
    public Boolean saveModuleContentContext(StationRequestBean reqBean, List<List<StationKvBean>> dataList) {
        return saveFsmBusiData(reqBean.getOrganizeCode(), getContextKey(reqBean), MesPcnExtConstWords.MODULE_CONTENT_CONTEXT, JSONObject.toJSONString(dataList));
    }

    @Override
    public List<List<StationKvBean>> getModuleContentContext(StationRequestBean reqBean) {
        String moduleContent = getFsmBusiData(reqBean.getOrganizeCode(), getContextKey(reqBean), MesPcnExtConstWords.MODULE_CONTENT_CONTEXT);
        return StringUtils.isEmpty(moduleContent) ? null : JSONObject.parseObject(moduleContent, new TypeReference<List<List<StationKvBean>>>() {});
    }

    //获取上下文加工结果
    @Override
    public String getProductResultContext(StationRequestBean reqBean) {
        return getFsmBusiData(reqBean.getOrganizeCode(), getContextKey(reqBean), MesPcnExtConstWords.PRODUCT_RESULT_CONTEXT);
    }

    //保存上下文加工结果    加工结果:成功/报废/可疑 【写入的值是变量类别代码对应的条码质量状态】【报废/可疑: 在保存逻辑条码时有特殊业务处理】
    @Override
    public Boolean saveProductResultContext(StationRequestBean reqBean, String productResult) {
        if (StringUtils.isEmpty(productResult)) return false;
        return saveFsmBusiData(reqBean.getOrganizeCode(), getContextKey(reqBean), MesPcnExtConstWords.PRODUCT_RESULT_CONTEXT, productResult);
    }

    //删除上下文加工结果
    @Override
    public void deleteProductResultContext(StationRequestBean reqBean) {
        deleteFsmBusiData(reqBean.getOrganizeCode(), getContextKey(reqBean), MesPcnExtConstWords.PRODUCT_RESULT_CONTEXT);
    }

    //获取上下文就绪信号
    @Override
    public String getReadySignalContext(StationRequestBean reqBean) {
        return getFsmBusiData(reqBean.getOrganizeCode(), getContextKey(reqBean), MesPcnExtConstWords.READY_SIGNAL_CONTEXT);
    }

    //保存上下文就绪信号   写1
    @Override
    public Boolean saveReadySignalContext(StationRequestBean reqBean) {
        return saveFsmBusiData(reqBean.getOrganizeCode(), getContextKey(reqBean), MesPcnExtConstWords.READY_SIGNAL_CONTEXT, CommonEnumUtil.TRUE_OR_FALSE.TRUE.getValueStr());
    }

    //删除上下文就绪信号
    @Override
    public void deleteReadySignalContext(StationRequestBean reqBean) {
        deleteFsmBusiData(reqBean.getOrganizeCode(), getContextKey(reqBean), MesPcnExtConstWords.READY_SIGNAL_CONTEXT);
    }

    //获取上下文发送允许加工指令执行结果
    @Override
    public String getSendProcessCmdContext(StationRequestBean reqBean) {
        return getFsmBusiData(reqBean.getOrganizeCode(), getContextKey(reqBean), MesPcnExtConstWords.SEND_PROCESS_CMD_CONTEXT);
    }

    //保存上下文发送允许加工指令执行结果    写接口逻辑的参考值
    @Override
    public Boolean saveSendProcessCmdContext(StationRequestBean reqBean, String value) {
        return saveFsmBusiData(reqBean.getOrganizeCode(), getContextKey(reqBean), MesPcnExtConstWords.SEND_PROCESS_CMD_CONTEXT, value);
    }

    //删除上下文发送允许加工指令执行结果
    @Override
    public void deleteSendProcessCmdContext(StationRequestBean reqBean) {
        deleteFsmBusiData(reqBean.getOrganizeCode(), getContextKey(reqBean), MesPcnExtConstWords.SEND_PROCESS_CMD_CONTEXT);
    }

    //获取上下文产出零件号
    @Override
    public List<MesEquipVariableCollectContext> getProductionPartNoContext(StationRequestBean reqBean) {
        String productionPartNoJson = getFsmBusiData(reqBean.getOrganizeCode(), getContextKey(reqBean), MesPcnExtConstWords.FIRST_MOULD_NO_CONTEXT);
        return !StringUtils.isEmpty(productionPartNoJson) ? JSONObject.parseArray(productionPartNoJson, MesEquipVariableCollectContext.class) : null;
    }

    //验证上下文产出零件号是否存在
    @Override
    public Boolean checkProductionPartNoIsExistContext(StationRequestBean reqBean) {
        String firstMouldNo = getFsmBusiData(reqBean.getOrganizeCode(), getContextKey(reqBean), MesPcnExtConstWords.FIRST_MOULD_NO_CONTEXT);
        return !StringUtils.isEmpty(firstMouldNo) ? true : false;
    }

    //保存上下文产出零件号  [JSON]List<MesEquipVariableCollectContext>
    @Override
    public Boolean saveProductionPartNoContext(StationRequestBean reqBean, List<MesEquipVariableCollectContext> productionPartNoList) {
        if (CollectionUtils.isEmpty(productionPartNoList)) return false;
        return saveFsmBusiData(reqBean.getOrganizeCode(), getContextKey(reqBean), MesPcnExtConstWords.FIRST_MOULD_NO_CONTEXT, JSONObject.toJSONString(productionPartNoList));
    }

    //删除上下文产出零件号
    @Override
    public void deleteProductionPartNoContext(StationRequestBean reqBean) {
        deleteFsmBusiData(reqBean.getOrganizeCode(), getContextKey(reqBean), MesPcnExtConstWords.FIRST_MOULD_NO_CONTEXT);
    }

    //获取上下文头道模具号
    @Override
    public MesEquipVariableCollectContext getFirstMouldNoContext(StationRequestBean reqBean) {
        String firstMouldNo = getFsmBusiData(reqBean.getOrganizeCode(), getContextKey(reqBean), MesPcnExtConstWords.FIRST_MOULD_NO_CONTEXT);
        return !StringUtils.isEmpty(firstMouldNo) ? JSONObject.parseObject(firstMouldNo, MesEquipVariableCollectContext.class) : null;
    }

    //验证上下文头道模具号是否存在
    @Override
    public Boolean checkFirstMouldNoIsExistContext(StationRequestBean reqBean) {
        String firstMouldNo = getFsmBusiData(reqBean.getOrganizeCode(), getContextKey(reqBean), MesPcnExtConstWords.FIRST_MOULD_NO_CONTEXT);
        return !StringUtils.isEmpty(firstMouldNo) ? true : false;
    }

    //保存上下文头道模具号  [JSON]MesEquipVariableCollectContext::equipVariableValue
    @Override
    public Boolean saveFirstMouldNoContext(StationRequestBean reqBean, MesEquipVariableCollectContext firstMouldNo) {
        if (null == firstMouldNo) return false;
        return saveFsmBusiData(reqBean.getOrganizeCode(), getContextKey(reqBean), MesPcnExtConstWords.FIRST_MOULD_NO_CONTEXT, JSONObject.toJSONString(firstMouldNo));
    }

    //删除上下文头道模具号
    @Override
    public void deleteFirstMouldNoContext(StationRequestBean reqBean) {
        deleteFsmBusiData(reqBean.getOrganizeCode(), getContextKey(reqBean), MesPcnExtConstWords.FIRST_MOULD_NO_CONTEXT);
    }

    //获取上下文一模多腔信息
    @Override
    public List<MesProductionPartContext> getProductionPartContext(StationRequestBean reqBean) {
        String productionPartJson = getFsmBusiData(reqBean.getOrganizeCode(), getContextKey(reqBean), MesPcnExtConstWords.PRODUCTION_PART_CONTEXT);
        return !StringUtils.isEmpty(productionPartJson) ? JSONObject.parseArray(productionPartJson, MesProductionPartContext.class) : null;
    }

    //验证上下文一模多腔信息是否存在
    @Override
    public Boolean checkProductionPartIsExistContext(StationRequestBean reqBean) {
        String productionPartJson = getFsmBusiData(reqBean.getOrganizeCode(), getContextKey(reqBean), MesPcnExtConstWords.PRODUCTION_PART_CONTEXT);
        return !StringUtils.isEmpty(productionPartJson) ? true : false;
    }

    //保存上下文一模多腔信息  [JSON]List<MesProductionPartContext>
    @Override
    public Boolean saveProductionPartContext(StationRequestBean reqBean, List<MesProductionPartContext> productionPartContextList) {
        if (CollectionUtils.isEmpty(productionPartContextList)) return false;
        return saveFsmBusiData(reqBean.getOrganizeCode(), getContextKey(reqBean), MesPcnExtConstWords.PRODUCTION_PART_CONTEXT, JSONObject.toJSONString(productionPartContextList));
    }

    //删除上下文一模多腔信息
    @Override
    public void deleteProductionPartContext(StationRequestBean reqBean) {
        deleteFsmBusiData(reqBean.getOrganizeCode(), getContextKey(reqBean), MesPcnExtConstWords.PRODUCTION_PART_CONTEXT);
    }

    //获取上下文零件数据信息
    @Override
    public Map<String, MesPart> getPartDataContext(StationRequestBean reqBean) {
        String partDataJson = getFsmBusiData(reqBean.getOrganizeCode(), getContextKey(reqBean), MesPcnExtConstWords.PART_DATA_CONTEXT);
        return !StringUtils.isEmpty(partDataJson) ? JSONObject.parseObject(partDataJson, new TypeReference<Map<String, MesPart>>() {}) : null;
    }

    //保存上下文零件数据信息  [JSON]Map<String, MesPart>
    @Override
    public Boolean savePartDataContext(StationRequestBean reqBean, Map<String, MesPart> partDataMap) {
        if (CollectionUtils.isEmpty(partDataMap)) return false;
        return saveFsmBusiData(reqBean.getOrganizeCode(), getContextKey(reqBean), MesPcnExtConstWords.PART_DATA_CONTEXT, JSONObject.toJSONString(partDataMap));
    }

    //删除上下文零件数据信息
    @Override
    public void deletePartDataContext(StationRequestBean reqBean) {
        deleteFsmBusiData(reqBean.getOrganizeCode(), getContextKey(reqBean), MesPcnExtConstWords.PART_DATA_CONTEXT);
    }

    //获取上下文模具号
    @Override
    public MesEquipVariableCollectContext getMouldNoContext(StationRequestBean reqBean) {
        String mouldNo = getFsmBusiData(reqBean.getOrganizeCode(), getContextKey(reqBean), MesPcnExtConstWords.MOULD_NO_CONTEXT);
        return !StringUtils.isEmpty(mouldNo) ? JSONObject.parseObject(mouldNo, MesEquipVariableCollectContext.class) : null;
    }

    //验证上下文模具号是否存在
    @Override
    public Boolean checkMouldNoIsExistContext(StationRequestBean reqBean) {
        String mouldNo = getFsmBusiData(reqBean.getOrganizeCode(), getContextKey(reqBean), MesPcnExtConstWords.MOULD_NO_CONTEXT);
        return !StringUtils.isEmpty(mouldNo) ? true : false;
    }

    //保存上下文模具号  [JSON]MesEquipVariableCollectContext::equipVariableValue
    @Override
    public Boolean saveMouldNoContext(StationRequestBean reqBean, MesEquipVariableCollectContext mouldNo) {
        if (null == mouldNo) return false;
        return saveFsmBusiData(reqBean.getOrganizeCode(), getContextKey(reqBean), MesPcnExtConstWords.MOULD_NO_CONTEXT, JSONObject.toJSONString(mouldNo));
    }

    //删除上下文模具号
    @Override
    public void deleteMouldNoContext(StationRequestBean reqBean) {
        deleteFsmBusiData(reqBean.getOrganizeCode(), getContextKey(reqBean), MesPcnExtConstWords.MOULD_NO_CONTEXT);
    }

    //获取上下文扫/读信息:加工单
    @Override
    public List<MesEquipVariableCollectContext> getScanWorkOrderNoContext(StationRequestBean reqBean) {
        String productSnJson = getFsmBusiData(reqBean.getOrganizeCode(), getContextKey(reqBean), MesPcnExtConstWords.SCAN_WORK_ORDER_NO_CONTEXT);
        return !StringUtils.isEmpty(productSnJson) ? JSONObject.parseArray(productSnJson, MesEquipVariableCollectContext.class) : null;
    }

    //验证上下文扫/读信息:加工单是否存在
    @Override
    public Boolean checkScanWorkOrderNoIsExistContext(StationRequestBean reqBean) {
        String productSnJson = getFsmBusiData(reqBean.getOrganizeCode(), getContextKey(reqBean), MesPcnExtConstWords.SCAN_WORK_ORDER_NO_CONTEXT);
        return !StringUtils.isEmpty(productSnJson) ? true : false;
    }

    //保存上下文扫/读信息:加工单  [JSON]List<MesEquipVariableCollectContext>
    @Override
    public Boolean saveScanWorkOrderNoContext(StationRequestBean reqBean, List<MesEquipVariableCollectContext> productSnList) {
        if (CollectionUtils.isEmpty(productSnList)) return false;
        return saveFsmBusiData(reqBean.getOrganizeCode(), getContextKey(reqBean), MesPcnExtConstWords.SCAN_WORK_ORDER_NO_CONTEXT, JSONObject.toJSONString(productSnList));
    }

    //删除上下文扫/读信息:加工单
    @Override
    public void deleteScanWorkOrderNoContext(StationRequestBean reqBean) {
        deleteFsmBusiData(reqBean.getOrganizeCode(), getContextKey(reqBean), MesPcnExtConstWords.SCAN_WORK_ORDER_NO_CONTEXT);
    }

    //获取上下文扫/读信息:主条码
    @Override
    public List<MesEquipVariableCollectContext> getScanProductSnContext(StationRequestBean reqBean) {
        String productSnJson = getFsmBusiData(reqBean.getOrganizeCode(), getContextKey(reqBean), MesPcnExtConstWords.SCAN_PRODUCT_SN_CONTEXT);
        return !StringUtils.isEmpty(productSnJson) ? JSONObject.parseArray(productSnJson, MesEquipVariableCollectContext.class) : null;
    }

    //验证上下文扫/读信息:主条码是否存在
    @Override
    public Boolean checkScanProductSnIsExistContext(StationRequestBean reqBean) {
        String productSnJson = getFsmBusiData(reqBean.getOrganizeCode(), getContextKey(reqBean), MesPcnExtConstWords.SCAN_PRODUCT_SN_CONTEXT);
        return !StringUtils.isEmpty(productSnJson) ? true : false;
    }

    //保存上下文扫/读信息:主条码  [JSON]List<MesEquipVariableCollectContext>
    @Override
    public Boolean saveScanProductSnContext(StationRequestBean reqBean, List<MesEquipVariableCollectContext> productSnList) {
        if (CollectionUtils.isEmpty(productSnList)) return false;
        return saveFsmBusiData(reqBean.getOrganizeCode(), getContextKey(reqBean), MesPcnExtConstWords.SCAN_PRODUCT_SN_CONTEXT, JSONObject.toJSONString(productSnList));
    }

    //删除上下文扫/读信息:主条码
    @Override
    public void deleteScanProductSnContext(StationRequestBean reqBean) {
        deleteFsmBusiData(reqBean.getOrganizeCode(), getContextKey(reqBean), MesPcnExtConstWords.SCAN_PRODUCT_SN_CONTEXT);
    }

    //获取上下文扫/读信息:装配件条码
    @Override
    public List<MesEquipVariableCollectContext> getScanAssemblySnContext(StationRequestBean reqBean) {
        String assemblySnJson = getFsmBusiData(reqBean.getOrganizeCode(), getContextKey(reqBean), MesPcnExtConstWords.SCAN_ASSEMBLY_SN_CONTEXT);
        return !StringUtils.isEmpty(assemblySnJson) ? JSONObject.parseArray(assemblySnJson, MesEquipVariableCollectContext.class) : null;
    }

    //验证上下文扫/读信息:装配件条码是否存在
    @Override
    public Boolean checkScanAssemblySnIsExistContext(StationRequestBean reqBean) {
        String assemblySnJson = getFsmBusiData(reqBean.getOrganizeCode(), getContextKey(reqBean), MesPcnExtConstWords.SCAN_ASSEMBLY_SN_CONTEXT);
        return !StringUtils.isEmpty(assemblySnJson) ? true : false;
    }

    //保存上下文扫/读信息:装配件条码  [JSON]List<MesEquipVariableCollectContext>
    @Override
    public Boolean saveScanAssemblySnContext(StationRequestBean reqBean, List<MesEquipVariableCollectContext> assemblySnList) {
        if (CollectionUtils.isEmpty(assemblySnList)) return false;
        return saveFsmBusiData(reqBean.getOrganizeCode(), getContextKey(reqBean), MesPcnExtConstWords.SCAN_ASSEMBLY_SN_CONTEXT, JSONObject.toJSONString(assemblySnList));
    }

    //删除上下文扫/读信息:装配件条码
    @Override
    public void deleteScanAssemblySnContext(StationRequestBean reqBean) {
        deleteFsmBusiData(reqBean.getOrganizeCode(), getContextKey(reqBean), MesPcnExtConstWords.SCAN_ASSEMBLY_SN_CONTEXT);
    }

    //获取进料主条码数据信息
    @Override
    public List<MesProductionPsInContext> getProductionPsInContext(StationRequestBean reqBean) {
        String productionPsInJson = getFsmBusiData(reqBean.getOrganizeCode(), getContextKey(reqBean), MesPcnExtConstWords.PRODUCTION_PS_IN_CONTEXT);
        return !StringUtils.isEmpty(productionPsInJson) ? JSONObject.parseArray(productionPsInJson, MesProductionPsInContext.class) : null;
    }

    //验证上下文进料主条码数据集合是否存在
    @Override
    public Boolean checkProductionPsInIsExistContext(StationRequestBean reqBean) {
        String productionPsInJson = getFsmBusiData(reqBean.getOrganizeCode(), getContextKey(reqBean), MesPcnExtConstWords.PRODUCTION_PS_IN_CONTEXT);
        return !StringUtils.isEmpty(productionPsInJson) ? true : false;
    }

    //保存上下文进料主条码数据信息集合  [JSON]List<MesProductionPsInContext>
    @Override
    public Boolean saveProductionPsInContext(StationRequestBean reqBean, List<MesProductionPsInContext> productionPsInContextList) {
        if (CollectionUtils.isEmpty(productionPsInContextList)) return false;
        return saveFsmBusiData(reqBean.getOrganizeCode(), getContextKey(reqBean), MesPcnExtConstWords.PRODUCTION_PS_IN_CONTEXT, JSONObject.toJSONString(productionPsInContextList));
    }

    //删除上下文进料主条码数据信息集合
    @Override
    public void deleteProductionPsInContext(StationRequestBean reqBean) {
        deleteFsmBusiData(reqBean.getOrganizeCode(), getContextKey(reqBean), MesPcnExtConstWords.PRODUCTION_PS_IN_CONTEXT);
    }

    //获取上下文产出条码数据信息集合
    @Override
    public List<MesProductionPsOutContext> getProductionPsOutContext(StationRequestBean reqBean) {
        String productionPsOutJson = getFsmBusiData(reqBean.getOrganizeCode(), getContextKey(reqBean), MesPcnExtConstWords.PRODUCTION_PS_OUT_CONTEXT);
        return !StringUtils.isEmpty(productionPsOutJson) ? JSONObject.parseArray(productionPsOutJson, MesProductionPsOutContext.class) : null;
    }

    //验证上下文产出条码数据信息集合是否存在
    @Override
    public Boolean checkProductionPsOutIsExistContext(StationRequestBean reqBean) {
        String productionPsOutJson = getFsmBusiData(reqBean.getOrganizeCode(), getContextKey(reqBean), MesPcnExtConstWords.PRODUCTION_PS_OUT_CONTEXT);
        return !StringUtils.isEmpty(productionPsOutJson) ? true : false;
    }

    //保存上下文产出条码数据信息集合  [JSON]List<MesProductionPsOutContext>
    @Override
    public Boolean saveProductionPsOutContext(StationRequestBean reqBean, List<MesProductionPsOutContext> productionPsOutContextList) {
        if (CollectionUtils.isEmpty(productionPsOutContextList)) return false;
        return saveFsmBusiData(reqBean.getOrganizeCode(), getContextKey(reqBean), MesPcnExtConstWords.PRODUCTION_PS_OUT_CONTEXT, JSONObject.toJSONString(productionPsOutContextList));
    }

    //删除上下文产出条码数据信息集合
    @Override
    public void deleteProductionPsOutContext(StationRequestBean reqBean) {
        deleteFsmBusiData(reqBean.getOrganizeCode(), getContextKey(reqBean), MesPcnExtConstWords.PRODUCTION_PS_OUT_CONTEXT);
    }

    //获取上下文产品加工规则信息集合
    @Override
    public List<MesProdRuleContext> getProdRuleDataContext(StationRequestBean reqBean) {
        String prodRuleNosortJson = getFsmBusiData(reqBean.getOrganizeCode(), getContextKey(reqBean), MesPcnExtConstWords.PROD_RULE_DATA_CONTEXT);
        return !StringUtils.isEmpty(prodRuleNosortJson) ? JSONObject.parseArray(prodRuleNosortJson, MesProdRuleContext.class) : null;
    }

    //验证上下文产品加工规则信息集合是否存在
    @Override
    public Boolean checkProdRuleDataIsExistContext(StationRequestBean reqBean) {
        String prodRuleNosortJson = getFsmBusiData(reqBean.getOrganizeCode(), getContextKey(reqBean), MesPcnExtConstWords.PROD_RULE_DATA_CONTEXT);
        return !StringUtils.isEmpty(prodRuleNosortJson) ? true : false;
    }

    //保存上下文产品加工规则信息集合  [JSON]List<MesProdRuleContext>
    @Override
    public Boolean saveProdRuleDataContext(StationRequestBean reqBean, List<MesProdRuleContext> prodRuleContextList) {
        if (CollectionUtils.isEmpty(prodRuleContextList)) return false;
        return saveFsmBusiData(reqBean.getOrganizeCode(), getContextKey(reqBean), MesPcnExtConstWords.PROD_RULE_DATA_CONTEXT, JSONObject.toJSONString(prodRuleContextList));
    }

    //删除上下文产品加工规则信息集合
    @Override
    public void deleteProdRuleDataContext(StationRequestBean reqBean) {
        deleteFsmBusiData(reqBean.getOrganizeCode(), getContextKey(reqBean), MesPcnExtConstWords.PROD_RULE_DATA_CONTEXT);
    }
    @Override
    public List<MesRawPartCharging> getMesRawPartChargingDataContext(StationRequestBean reqBean) {
        String mesRawPartChargingDataJson = getFsmBusiData(reqBean.getOrganizeCode(), getContextKey(reqBean), "RAW_PART_CHARGING_DATA_CONTEXT");
        return !StringUtils.isEmpty(mesRawPartChargingDataJson) ? JSONObject.parseArray(mesRawPartChargingDataJson, MesRawPartCharging.class) : null;
    }

    @Override
    public Boolean saveMesRawPartChargingDataContext(StationRequestBean reqBean, List<MesRawPartCharging> mesRawPartChargingList) {
        if (CollectionUtils.isEmpty(mesRawPartChargingList)) return false;
        return saveFsmBusiData(reqBean.getOrganizeCode(), getContextKey(reqBean), "RAW_PART_CHARGING_DATA_CONTEXT", JSONObject.toJSONString(mesRawPartChargingList));
    }

    @Override
    public void deleteMesRawPartChargingDataContext(StationRequestBean reqBean) {
        deleteFsmBusiData(reqBean.getOrganizeCode(), getContextKey(reqBean), "RAW_PART_CHARGING_DATA_CONTEXT");
    }

}
